@@ -3,7 +3,7 @@ extern crate core;
 #[allow(clippy::missing_errors_doc)]
 
 pub mod wtcsv;
-mod diff;
+pub mod diff;
 
 const DELIMITER: char = ';';
 const RECORD_SEP: char  = '\n';
